/*-
 * %%----------------------------------------------------------------------------------------------
 * Solidify Framework - Solidify Controller - SolidifyProperties.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.unige.solidify.config;

import java.net.URL;
import java.net.URLClassLoader;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.stereotype.Component;

import ch.unige.solidify.service.GitInfoProperties;

@Component
@ConfigurationProperties(prefix = "solidify")
public class SolidifyProperties {
  private static final Logger log = LoggerFactory.getLogger(SolidifyProperties.class);

  // ****************
  // ** Properties **
  // ****************
  @Autowired
  private GitInfoProperties gitInfoProperties;

  @Value("${spring.application.name}")
  private String applicationName;

  @Value("${server.servlet.application-display-name:Default Server Display Name}")
  private String serverDisplayName;

  private Email email = new Email();
  private Orcid orcid = new Orcid();

  // *********************
  // ** Getters/Setters **
  // *********************
  public static Logger getLog() {
    return log;
  }

  public String getApplicationName() {
    return this.applicationName;
  }

  public String getApplicationVersion() {
    return this.gitInfoProperties.getBuild().getVersion();
  }

  public GitInfoProperties getGitInfoService() {
    return this.gitInfoProperties;
  }

  public String getServerDisplayName() {
    return this.serverDisplayName;
  }

  public void logClassPath() {
    final ClassLoader cl = ClassLoader.getSystemClassLoader();
    final URL[] urls = ((URLClassLoader) cl).getURLs();
    log.info("List CLASSPATH");
    for (final URL url : urls) {
      log.info(url.getFile());
    }
  }

  public Email getEmail() {
    return this.email;
  }

  public void setEmail(Email email) {
    this.email = email;
  }

  public Orcid getOrcid() {
    return orcid;
  }

  public void setOrcid(Orcid orcid) {
    this.orcid = orcid;
  }

  // *************************
  // ** Embedded Properties **
  // *************************

  public static class Email {
    private String senderAddress;
    private String[] ccList = {};
    private String[] bccList = {};
    private String externalTemplates;
    private String internalTemplates = "email-templates/";

    public String getSenderAddress() {
      return this.senderAddress;
    }

    public void setSenderAddress(String senderAddress) {
      this.senderAddress = senderAddress;
    }

    public String getExternalTemplates() {
      return this.externalTemplates;
    }

    public String[] getCcList() {
      return this.ccList;
    }

    public void setCcList(String[] ccList) {
      this.ccList = ccList;
    }

    public String[] getBccList() {
      return this.bccList;
    }

    public void setBccList(String[] bccList) {
      this.bccList = bccList;
    }

    public void setExternalTemplates(String externalTemplates) {
      this.externalTemplates = externalTemplates;
    }

    public String getInternalTemplates() {
      return this.internalTemplates;
    }

    public void setInternalTemplates(String internalTemplates) {
      this.internalTemplates = internalTemplates;
    }
  }

  public static class Orcid {
    private String clientId;
    private String clientSecret;

    public String getClientId() {
      return clientId;
    }

    public void setClientId(String clientId) {
      this.clientId = clientId;
    }

    public String getClientSecret() {
      return clientSecret;
    }

    public void setClientSecret(String clientSecret) {
      this.clientSecret = clientSecret;
    }

  }
}
