/*-
 * %%----------------------------------------------------------------------------------------------
 * Solidify Framework - Solidify Controller - SolidifyDbLockError.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.unige.solidify.repository;

import javax.servlet.http.HttpServletRequest;

import org.springframework.boot.web.servlet.error.ErrorAttributes;
import org.springframework.dao.CannotAcquireLockException;
import org.springframework.dao.ConcurrencyFailureException;
import org.springframework.http.HttpStatus;
import org.springframework.orm.ObjectOptimisticLockingFailureException;

public class SolidifyDbLockError extends SolidifyError {

  boolean isOptimisticLock;

  public SolidifyDbLockError() {
  }

  public SolidifyDbLockError(ErrorAttributes errorAttributes, HttpServletRequest request,
          ConcurrencyFailureException e) {
    super(errorAttributes, request);
    if (e instanceof CannotAcquireLockException) {
      this.isOptimisticLock = false;
      this.setStatus(HttpStatus.PRECONDITION_FAILED);
    }
    if (e instanceof ObjectOptimisticLockingFailureException) {
      this.isOptimisticLock = true;
      this.setStatus(HttpStatus.PRECONDITION_FAILED);
    }
  }

  public boolean getOptimisticLock() {
    return this.isOptimisticLock;
  }

  public void setOptimisticLock(boolean optimisticLock) {
    this.isOptimisticLock = optimisticLock;
  }

}
