/*-
 * %%----------------------------------------------------------------------------------------------
 * Solidify Framework - Solidify Index Controller - IndexResourceService.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.unige.solidify.service;

import java.util.List;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;

import ch.unige.solidify.model.index.IndexMetadata;
import ch.unige.solidify.rest.FacetPage;
import ch.unige.solidify.rest.FacetRequest;
import ch.unige.solidify.rest.SearchCondition;

public abstract class IndexResourceService<I, T extends IndexMetadata> extends AbstractResourceService {

  public Page<T> findAll(I indexName, T search, Pageable pageable) {
    final List<T> list = this.findAll(indexName, search);
    final long start = pageable.getOffset();
    final long end = (start + pageable.getPageSize()) > list.size() ? list.size()
            : (start + pageable.getPageSize());
    return new PageImpl<>(list.subList((int) start, (int) end), pageable, list.size());
  }

  // Default Functions
  public Page<T> search(I indexName, String query, Pageable pageable) {
    return this.search(indexName, query, (String) null, null, null, null, pageable);
  }

  public FacetPage<T> search(I indexName, String query, String conditions, List<FacetRequest> facetRequests, Pageable pageable) {
    Page<T> page = this.search(indexName, query, (String) null, null, null, null, pageable);

    if (page instanceof FacetPage) {
      return (FacetPage<T>) page;
    } else {
      return new FacetPage<>(page.getContent(), null, page.getPageable(), page.getTotalElements());
    }
  }

  public Page<T> search(I indexName, String query, String set, String field,
          String from, String to, Pageable pageable) {
    return this.findAll(indexName, null, pageable);
  }

  public FacetPage<T> search(I indexName, String query, List<FacetRequest> facetRequests, String set, String field,
          String from, String to, String filter, Pageable pageable) {
    Page<T> page = this.findAll(indexName, null, pageable);

    if (page instanceof FacetPage) {
      return (FacetPage<T>) page;
    } else {
      return new FacetPage<>(page.getContent(), null, page.getPageable(), page.getTotalElements());
    }
  }

  public Page<T> searchRange(I indexName, String field, String from, String to,
          Pageable pageable) {
    return this.findAll(indexName, null, pageable);
  }

  public FacetPage<T> search(I indexName, List<SearchCondition> conditions, List<FacetRequest> facetRequests, Pageable pageable) {
    return this.search(indexName, null, null, facetRequests, pageable);
  }

  public abstract T findOne(I indexName, String id);

  public abstract List<T> findAll(I indexName, T search);

  public abstract T save(I indexName, T t);

  public abstract T update(I indexName, T t);

  public abstract boolean delete(I indexName, T t);

  public abstract void deleteList(I indexName, List<String> ids);

  // Define Database name ~ index name
  public abstract String getIndexName(I indexName);
}
